# -*- coding: utf-8 -*-
"""NIPY is a python project for analysis of structural and functional neuroimaging data."""
from .model import FitGLM, EstimateContrast
from .preprocess import ComputeMask, SpaceTimeRealigner
from .utils import Similarity
